/***************************************************************************
 *                                                                         *
 *   TCPIP.H                                                               *
 *                                                                         *
 *   Copyright (c) 1994-1996 Galacticomm, Inc.    All Rights Reserved.     *
 *                                                                         *
 *   TCP/IP for the Internet Connectivity Option for Worldgroup,           *
 *   based upon Pacific Softworks' Fusion, or Ipswitch's Piper/IP          *
 *   TCP/IP protocol kernel.                                               *
 *                                                                         *
 *   Ipswitch's Piper/IP support          - RNStein  5/25/94               *
 *   Pacific Softworks' Fusion port       - RNStein  4/11/95               *
 *                                                                         *
 ***************************************************************************/

#define ICO2                       /* for compatibility with ifdef's       */

#ifndef PACSOFT
#ifndef IPSWITCH
#error Must define either "PACSOFT" or "IPSWITCH"
#endif
#endif

#ifdef PACSOFT
#ifdef IPSWITCH
#error Must define either "PACSOFT" or "IPSWITCH" but not both
#endif
#endif

/*--- Pacific Softworks-specific include files that are used for ICO code ---*/
#ifdef PACSOFT
#include "fusion.h"
#include "llconfig.h"
#endif

/*--- Ipswitch-specific includes, etc., that are used for ICO code ---*/
#ifdef IPSWITCH
#include "sys\xdefs.h"
#include "sys\types.h"
#include "sys\uio.h"
#include "sys\socket.h"
#include "sys\mbuf.h"
#include "net\if.h"
#include "arpa\nameser.h"
#include "netinet\in.h"
#include "nlist.h"
#include "netdb.h"
#include "sockprot.h"
#undef  INADDR_NONE                /* avoid Borland compiler's warning     */
#define INADDR_NONE 0xFFFFFFFFL    /* indeterminate IP address (ipaddrb)   */
#define EIO      5                 /* error codes missing from sys/xdefs.h */
#define ENXIO    6
#define ENOTDIR 20
#define ENOTTY  25
#endif

/* datstm.h must always be included, usually via brkthu.h via majorbbs.h   */

/*--- General constants and variables ---*/
#define INET_ADDR_ERR 0xFFFFFFFFL  /* inet_addr() return value: bad syntax */
#define tcpip_errno (*perrno)      /* TCP/IP kernel's err number           */
extern int _soisfd;                /* for distinct file/socket handle sets */
extern int h_errno;                /* for gethtbyname() error codes        */
extern int *perrno;                /* pointer to TCP/IP kernel's err number*/


struct svrinf {                    /* Registered Server information        */
     char *name;                   /* name of svr (eg "FTP") max len 16    */
     struct svrinf *next;          /* link to next server, (NULL=end)      */
     int port;                     /* TCP port number (e.g. 23=Telnet)     */
     int backlog;                  /* max # of colliding incoming calls    */
     int svrskt;                   /* server's bind() & listen() socket #  */
     void (*incall)(               /* incoming call handler (socket=clskt) */
          int gotchn);             /* 1=chan (curusr) assigned, 0=not avail*/
};

#define TCPOSZ 128                 /* size of TCP/IP output staging buffer */

struct tcpipinf {                  /* channel-specific TCP/IP info         */
     struct bufstm outsnk;         /* output sink (MUST BE FIRST FIELD)    */
     char outstg[TCPOSZ];          /* output staging buffer                */
     struct datstm *cdidst;        /* GSBL's sink for input byte hose      */
     struct svrinf *server;        /* server handling channel (NULL=none)  */
     struct in_addr inaddr;        /* Client's 32-bit internet address     */
     int port;                     /* Client's remote TCP port number      */
     int socket;                   /* socket to client (from accept())     */
     int unum;                     /* user number (sink may need to know)  */
     char flags;                   /* see below                            */
     long bytcnt;                  /* extra bytes xferred (eg data port)   */
     unsigned long tckcon;         /* when connection was made             */
     char spare[200-187];          /* (round out structure to 200 bytes)   */
};                                 /* (Structure cannot exceed 255 bytes.) */
                                   /* (Changing size of this structure     */
                                   /* requires all add-ons be recompiled.) */

/*--- tcpip info flags --- */
#define TCPDSC 0x01                /* disconnect in progress               */

extern FILE *tcpmg;                /*--- galtcpip.mcv stuff             ---*/
extern char *ipaddr;               /* this Worldgroup server's IP address  */
extern unsigned long ipaddrb;      /* IPADDR, binary form, netw byte order */
extern unsigned long iproutb;      /* IPROUT, (router IP), netw byte order */
extern unsigned long netmaskb;     /* NETMASK, (perh derived from IPADDR)  */
extern unsigned netmtu;            /* NETMTU, MTU for your network         */
extern char audinc;                /* Audit Trail record incoming calls?   */
extern char audrej;                /* Audit Trail reject calls due to busy?*/
extern char audtrm;                /* Audit Trail termination of calls?    */
extern int sndwin;                 /* SO_SNDBUF socket option              */
extern int rcvwin;                 /* SO_RCVBUF socket option              */
extern char *hostdeny;             /* file of IP's to deny access          */

                                   /*--- other public vars               ---*/
extern struct svrinf *svrhead;     /* head of linked svr list (NULL=none)  */
extern struct tcpipinf *tcpipinf;  /* array[nterms] of TCP/IP-channel info */
extern struct sockaddr_in claddr;  /* client address (when incall()'ing)   */
extern int clskt;                  /* socket to client (implicit parameter */
                                   /* to server's incall() routine)        */
extern int nfyskt;                 /* socket you're being notified about   */
extern int sndact;                 /* sndmgr() records no. of bytes sent   */
extern char *tcposb;               /* PACSOFT: global output staging buffer*/
extern unsigned long selewb;       /* count select()/nselect() EWOULDBLOCKs*/
extern unsigned long accewb;       /* count accept() EWOULDBLOCKs          */

extern
char tcpip_kid[];                  /* TCP/IP kernel implementation ID      */

/*--- socket event handling map ---*/
struct sktmap {                    /* socket event handlers: RECV & SEND   */
     void (*evthdl)();             /* event handler routine (NULL=none)    */
     void *ptr;                    /* app-specific data pointer            */
};
                                   /* tnftype parm of sktnfy() & sktcnc(): */
#define TNFRECV 0                  /* Notify of recv() opportunity         */
#define TNFSEND 1                  /* Notify of send() opportunity         */
#define TNFACCP 2                  /* Notify of accept() opportunity       */
#define TNFCONN 3                  /* Notify of connect() completion       */
#define TNFQUAN 4                  /* number of possible values of tnftype */

                                   /* inner (1st) index in sktmap array    */
#define TNFRA 0                    /* socket map for RECV and ACCEPT       */
#define TNFSC 1                    /* socket map for SEND and CONNECT      */
#define TNFNUM 2                   /* possible indices in sktmap[][] array */

extern
int tnfidx[TNFQUAN];               /* map tnftype -> inner sktmap[][] index*/
extern
int tnfclosed;                     /* notifies event handler of peer close */
                                   /* (set before sktnfy()'s evthdl is     */
                                   /* called, should be 0 all other times) */

extern
BOOL (*hstalw)(                    /* host allowable vector (TRUE=allowed) */
     unsigned long hostip);
extern
int ijreas;                        /* elaborates on ->incall(0)            */

                                   /* reasons why incoming call rejected   */
#define IJRFULL 1                  /* no room for incoming call            */
#define IJRSHUT 2                  /* server shutting down                 */
#define IJRDENY 3                  /* (*hstalw)() rejected                 */
#define IJRNONE 0                  /* not rejected:  ->incall(1)           */

#ifdef PACSOFT
     #define BASE_NOTIFY (EXCEPT_NOTIFY \
                      +RSHUTDOWN_NOTIFY \
                      +WSHUTDOWN_NOTIFY)
     extern
     int selflags[TNFQUAN];        /* map tnftype -> nselect()'s flag masks*/
     extern
     int numnfy;                   /* number of sockets interested in      */
     extern
     struct sel *sktsel;           /* sockets of interest (for nselect())  */
#endif

/*--- tcpdial() and udpdial() return values ---*/
#define DLERRS -1                  /* socket() error (see tcpip_errno)     */
#define DLERRI -2                  /* ioctl() error (see tcpip_errno)      */
#define DLERRR -3                  /* setsockopt(SO_REUSEADDR) error       */
#define DLERRB -5                  /* bind() error (see tcpip_errno)       */
#define DLCNOW 6                   /* success, socket available now!       */

/*--- more tcpdial() return values (not udpdial()) ---*/
#define DLERRC -7                  /* connect() error (see tcpip_errno)    */
#define DLCING 8                   /* connect() deferred, select for send  */

/*--- GCDI interrupts (candidates for somewhere more general-purpose) ---*/
#define dealwith_cdi_interrupts   int savifl
#define allow_cdi_interrupts      (savifl=__allow_cdi_interrupts())
#define prevent_cdi_interrupts    (savifl=__prevent_cdi_interrupts())
#define restore_cdi_interrupts    (__restore_cdi_interrupts(savifl))

void EXPORT
init__tcpip(void);                 /* Initialize TCP/IP (ICO add-ons MUST  */
                                   /* call at top of their init__xxx()'s)  */

int
regtcpsvr(                         /* register a TCP server                */
char *name,                        /* name of service (e.g. FTP, Telnet)   */
int port,                          /* port number (e.g. 23=telnet, 21=ftp) */
int backlog,                       /* max # of colliding incoming calls    */
void (*incall)(                    /* incoming call handler                */
     int unum));                   /* user number (usrnum), or -1=unavail  */
                                   /* returns 1=up ok, 0=can't             */

void
rejinc(                            /* reject an incoming TCP/IP call       */
char *svr,                         /* name of that server                  */
char *why);                        /* reason for rejection                 */

unsigned int
tcpmit(                            /* TCP/IP moveit(), channel output sink */
struct datstm *dsp,                /* (pointer to sink's datstm structure) */
char *srcloc,                      /* source specifies the location        */
unsigned int nwant);               /* source wants sink to move this many  */

unsigned int                       /*   returns room in send queue         */
tcpskw(                            /* TCP/IP snkwin() (relies on sendba()) */
struct datstm *dsp);               /*   pointer to sink's datstm structure */

void
tcpdmv(                            /* TCP/IP didmov() (relies on sendba()) */
struct datstm *dsp,                /*   pointer to sink's datstm structure */
unsigned nactual);                 /*   number of bytes moved to tcposb    */

void
tcpinc(                            /* read socket data into GCDI channel   */
struct tcpipinf *tip);             /* user's TCP/IP channel info           */
                                   /* application:  sktnfy(,,tcpinc,tip,)  */

void
tcpsnd(                            /* try to send staging buffer contents  */
struct tcpipinf *tip);             /* TCP/IP session info                  */

int
sndmgr(                            /* manage send()ing from a buffer       */
char *buffer,                      /* buffer address                       */
unsigned int *bufcnt,              /* where buffer count is stored         */
int socket);                       /* socket handle                        */
                                   /* returns 1=not done, 0=done, -1=error */
                                   /* sndact implicit output: # bytes sent */

void
audtcptfc(                         /* Audit time and traffic for TCP server*/
char *title,                       /* first line of Audit Trail message    */
long bytcnt);                      /* number of bytes traffic              */

int
tcpdial(                           /* initiate a TCP connection            */
struct in_addr inaddr,             /* host IP address                      */
unsigned int port,                 /* port number (network byte order)     */
unsigned int fromport,             /* local port number (0=ephemeral)      */
int *psocket);                     /* where to store socket handle (should */
                                   /* be -1 whenever this socket not open) */
                                   /* see DLXXXX return values in TCPIP.H  */

int
udpdial(                           /* initiate a UDP connection            */
unsigned int fromport,             /* local port number (0=ephemeral)      */
int *psocket);                     /* where to store socket handle (should */
                                   /* be -1 whenever this socket not open) */
                                   /* see DLXXXX return values in TCPIP.H  */

void
nonblock(                          /* set socket to non-blocking           */
int skt);                          /* socket descriptor                    */

void
oobinline(                         /* mix OOB data with regular data       */
int skt);                          /* socket descriptor                    */

unsigned long
recvbw(                            /* recv() bytes waiting (ret number of) */
int skt);                          /* socket descriptor                    */

unsigned int
sendba(                            /* send() bytes able (ret number of)    */
int skt);                          /* socket descriptor                    */

void
rwnmgr(                            /* receive window manager               */
int socket,                        /* socket handle                        */
int nroom,                         /* max room to advertise (- means 0)    */
int *curwin);                      /* current advertised window variable   */
                                   /* rwnmgr() can only SHRINK window size.*/

int
rwnadj(                            /* receive window adjuster              */
int socket,                        /* socket handle                        */
int bufsiz,                        /* incoming buffer capacity when empty  */
int bufavl,                        /* incoming buffer capacity now         */
int hyster,                        /* hysteresis on window adjustment      */
int *curwin);                      /* current advertised window variable   */
                                   /* returns 1=adjusted, 0=left alone     */
                                   /* An empty buffer means immediately    */
                                   /* advertise max window, else there's   */
                                   /* hysteresis on receive window growth. */
                                   /* rwnadj() can only GROW window size.  */

int
setsndwin(                         /* set socket's send window size        */
int socket,                        /* socket handle                        */
int swin);                         /* send window (e.g. sndwin)            */
                                   /* returns 1=ok, 0=error (tcpip_errno)  */

int
setrcvwin(                         /* set socket's receive window size     */
int socket,                        /* socket handle                        */
int rwin);                         /* receive window (e.g. rcvwin)         */
                                   /* returns 1=ok, 0=error (tcpip_errno)  */

void
fd_scanbeg(                        /* expr1 in fd_set scanning for(;;) loop*/
fd_set *fdsp);

int
fd_scanmor(void);                  /* expr2 in fd_set scanning for(;;) loop*/

void
fd_scannxt(void);                  /* expr3 in fd_set scanning for(;;) loop*/

void
fd_setor(                          /* bit-wise OR two fd_set's together    */
struct fd_set *oree,               /* input and output                     */
struct fd_set *orer);              /* input                                */

void
fd_setand(                         /* bit-wise AND two fd_set's together   */
struct fd_set *andee,              /* input and output                     */
struct fd_set *ander);             /* input                                */

int                                /*   returns 1=valid, 0=non-class A,B,C */
dftnetmask(                        /* default network mask                 */
unsigned long ipaddr,              /*   IP addresss (network byte order)   */
unsigned long *netmask);           /*   where to put netmask (n byte order)*/

int                                /*   returns 1=valid, 0=not             */
chknetmask(                        /* is netmask valid?                    */
unsigned long netmask);            /*   netmask, network byte order        */
                                   /*   Netmask must allow at least 4 hosts*/
                                   /*   and in binary start with 1's and   */
                                   /*   end with 0's.                      */

int
sndop(                             /* send an option string, all or nothing*/
struct tcpipinf *tip,              /* TCP/IP info for channel              */
char *bytes,                       /* binary string of bytes (may incl \0) */
int nbytes);                       /* number of bytes                      */
                                   /* returns 0=no room, 1=ok              */

void
sktnfy(                            /* request notification on socket event */
int tnftype,                       /* TNFRECV, TNFSEND, TNFACCP, TNFCONN   */
int skt,                           /* socket number                        */
void (*evthdl)(),                  /* event handler routine                */
void *ptr,                         /* app-specific ptr (passed to evthdl)  */
int unum);                         /* associated user number (curusr() set */
                                   /* before evthdl) or -1=none            */

/* NOTE:  Do not request notification for both TNFSEND and TNFCONN on the same
socket at the same time, unless the last 3 parameters to sktnfy() are
identical with each request.  (In practice, interest in sendability and
connection-established are mutually exclusive anyway.)  Same goes for TNFRECV
and TNFACCP (which are also mutually exclusive interests).  So, for example,
when switching from TNFCONN interest to TNFSEND interest, be sure to
sktcnc(TNFCONN,...) before you sktnfy(TNFSEND...).  Under the Ipswitch stack,
sendability and connect-done are indistinguishable, as well as receivability
and accept-readiness.  Under Fusion they are distinct conditions.  Failure to
follow the above rules may have different penalties on different stacks.  */

void
sktcnc(                            /* cancel notification on socket events */
int tnftype,                       /* TNFRECV, TNFSEND, TNFACCP, TNFCONN   */
int skt);                          /* socket number                        */

/* All sktnfy() calls must have a corresponding sktcnc() call, except calling
clsskt() implicitly cancels all requests for the socket. */

void
clssvr(                            /* close a server's socket to his client*/
int unum,                          /* user number (or -1=not applicable)   */
int skt,                           /* socket                               */
int port);                         /* server's port number (for error msg) */

int
clsskt(                            /* close a socket                       */
int skt);                          /* socket number                        */
                                   /* returns 1=worked 0=failed (see errno)*/

void
byetcp();                          /* log-off a TCP/IP user w/message      */
                                   /* same parameters as prfmsg()          */
                                   /* usrnum is an implicit input          */
                                   /* (for backward compatibility only;    */
                                   /* for new code, use byendl())          */

int                                /*   value to pass to iskclose()        */
iskopen(void);                     /* permit GCDI channel input sink use   */

void
iskclose(                          /* end of GCDI channel input sink use   */
int oldicx);                       /*   from corresponding iskopen()       */

int
__allow_cdi_interrupts(void);      /* guts of allow_cdi_interrupts macro   */

int
__prevent_cdi_interrupts(void);    /* guts of prevent_cdi_interrupts macro */

void
__restore_cdi_interrupts(          /* guts of restore_cdi_interrupts macro */
int savifl);

void
cdi_dmbufs(                        /* dmbufs() with interrupts enabled     */
struct datstm far *dsp,            /* DataStream structure (inside bufstm) */
unsigned int nactual);             /* this many were moved by the source   */

void
svrcatast(                         /* system-fatal error with TCP/IP svr   */
char *type,                        /* such as "BIND" or "LISTEN"           */
int port);                         /* TCP port number                      */

void
svrfail(                           /* session-fatal error with TCP/IP svr  */
int unum,                          /* user number (or -1=not applicable)   */
char *type,                        /* such as "SEND" or "RECEIVE"          */
int port);                         /* TCP port number                      */

void
tcpfail();                         /* session-fatal error with TCP/IP      */
                                   /* (see TCPIP.C for parameters)         */

