/***************************************************************************
 *                                                                         *
 *   WEBD.H                                                                *
 *                                                                         *
 *   Copyright (c) 1994-1996 Galacticomm, Inc.    All Rights Reserved.     *
 *                                                                         *
 *   World-Wide Web (WWW) server for access to HTML pages.                 *
 *                                                                         *
 *   Ref HTTP/1.0 draft of 3/8/95:                                         *
 *        ftp://ds.internic.net/internet-drafts/                           *
 *        draft-ietf-http-v10-spec-00.txt                                  *
 *                                                                         *
 *   Ref HTML - 2.0 draft of 6/16/95                                       *
 *        ftp://ds.internic.net/internet-drafts/                           *
 *        draft-ietf-html-spec-04.txt                                      *
 *                                                                         *
 *                                              - R. Stein  8/5/94         *
 *   Clickable image maps, statistics logging   - C. Yap    6/95           *
 *   HTTP 1.0, interceptable vectors, forms     - R. Stein  7/95           *
 *                                                                         *
 ***************************************************************************/

#if defined(UNIX)
#if !defined(ASSERT)
#define ASSERT(x)
#define ASSERTM(x,y)
#endif
#define cr2lf(x)        strstp(x,'\r')
#else
#define cr2lf(x)        x
#endif

#define DFTWEBPORT 80              /* default TCP port for Web server      */
#define WEBNAME "World-Wide Web"   /* name of Web server                   */
#define WEBBACKLOG 5               /* listen() backlog                     */
#define URISIZ 1024                /* max size of a URI                    */
#define MAXVERT 100                /* max vertices in a polygon            */
#define MINREQBUF 2048             /* minimum guaranteed room at httpreqbuf*/
#define BDYMAX 100000UL            /* default max body size (in bdyfil)    */

struct webdusr {                   /* Web server channel information       */
     unsigned int reqcnt;          /* number of bytes stored in httpreqbuf */
     unsigned int hdrsiz;          /* size of req line & header, incl \n's */
     unsigned int reqsiz;          /* size of entire request line, incl \n */
     unsigned int flags;           /* see below                            */
     unsigned int httpver;         /* HTTP version number * 100 (100=1.0)  */
     unsigned int sttime;          /* 16 Hz activity timer                 */
     unsigned long conlen;         /* Content-Length                       */
     unsigned long concnt;         /* count of content body written to file*/
     uchar *contyp;                /* Content-Type                         */
     uchar *userag;                /* User-Agent                           */
     uchar *method;                /* request method (GET, PUT, etc.)      */
     uchar *uri;                   /* request URI                          */
     uchar *vertxt;                /* version text, or "" if none          */
     uchar *refer;                 /* Referer                              */
     uchar *auth;                  /* Authorization                        */
     unsigned short imsdat;        /* If-Modified-Since, date part         */
     unsigned short imstim;        /* If-Modified-Since, time part         */
     unsigned long bytcnt;         /* total bytes transferred to user      */
     FILE *rspfp;                  /* response coming from a file          */
     FILE *reqfp;                  /* request body stored in temporary file*/
};                                 /* ...httphdr uses up rest of VDA       */
                                   /* ...which is at least HTTPHDRSIZ bytes*/

#define WFLERR 0x0001              /* error page being sent                */
#define WFLIMS 0x0002              /* If-Modified-Since request received   */

#define webdptr ((struct webdusr *)vdaptr)
#define httpreqbuf (vdaptr+sizeof(struct webdusr))
#define maxreqbuf (vdasiz-sizeof(struct webdusr))

                                   /*--- Web server substate codes      ---*/
#define WEBHDR 0                   /* collect header of request to httphdr */
#define WEBBDY 1                   /* record body of request to a file     */
#define WEBHDL 2                   /* handle request                       */
#define WEBFIL 3                   /* dump file to client as response body */

extern
char *webroot;                     /* path prefix to all Web HTML pages    */
extern
char *imgroot;                     /* path prefix for image maps           */
extern
char *dftfil;                      /* default file for "GET /" response    */
extern
char *imgpfix;                     /* prefix for (*hdlimgmap)() i'ceptors  */
                                   /* always begins and ends with "/"      */
extern
char *noclick;                     /* pseudo-URL for image maps: ignore clk*/
extern
char *webd_version;                /* Web server version string (for tvar) */
extern
int webtzone;                      /* minutes behind GMT (240=EDT)         */

extern                             /*   returns 1=ok, 0=byeweb() called    */
short (*prswebhdr)(void);          /* parse web request header             */
                                   /*   see webdptr->reqsiz and hdrsiz     */
                                   /*   (first vector called for new req)  */

extern                             /*   returns 1=handled, 0=not recognized*/
short (*prshdrlin)(                /* handle header line of incoming req   */
     char *hdrlin);                /*   for each header line (not req line)*/

extern
void (*hdlwebreq)(                 /* request interceptor                  */
     char *uri);                   /*   URI following method in req line   */

extern                             /*   ret 1=handled, 0=404 err (no found)*/
short (*hdlwebget)(                /* GET request interceptor              */
     char *uri);                   /*   URI following 'GET' in request line*/

extern                             /*   ret 1=mapped, 0=no action, -1=error*/
int (*hdlimgmap)(                  /* Image map mouse-click interceptor    */
     char *mapnam,                 /*   Image map file name (after imgpfix)*/
     unsigned int xt,              /*   pixel X of mouse click             */
     unsigned int yt,              /*   pixel Y of mouse click             */
     char **newurl);               /*   where to put pointer to mapped URL */
                                   /*   (useful when returning 1 only)     */

extern                             /*   returns no. bytes sent as rsp body */
int (*hdlrspbdy)(                  /* response body bytes en route client  */
     char *buffer,                 /*   bytes to send in response body     */
     int nbytes);                  /*   number of bytes                    */
                                   /*   CAUTION: buffer may be vdatmp      */

extern
char *bdyfil;                      /* temporary file for incom. Entity Body*/
extern
unsigned long bdymax;              /* maximum size of body (in bdyfil)     */

void EXPORT
init__webd(void);                  /* Initialization                       */

short                              /*   ret 1=handled, 0=not recognized    */
mapwebget(                         /* NCSA-style image map file interpreter*/
char *uri);                        /* URI from client                      */

short                              /*   ret 1=handled, 0=404 err (no found)*/
filwebget(                         /* file-retrieval form of GET request   */
char *uri);                        /*   URI (file path rel to WEBROOT{})   */
                                   /*   (corrupts vdatmp)                  */

short                              /*   ret 1=bad (byeweb() called), 0=ok  */
badfspec(                          /* Is URI-based filespec dangerous?     */
char *uri);                        /*   URI or filespec based on a URI     */

short                              /*   returns 1=match, 0=not             */
hdrstg(                            /* parse a named token                  */
char *source,                      /*   line, eg 'Content-Type: text/html' */
char *token,                       /*   token name, eg 'Content-Type:'     */
char **var);                       /*   where to put value, eg 'text/html' */

unsigned long
udif2(                             /* square of difference of two u-ints   */
unsigned int a,
unsigned int b);

char *
webpth(                            /* format path name of web page         */
char *pagnam);                     /* page name (can start with "/" or "\")*/
                                   /* returns "" if combination too long   */

char *
imgpth(                            /* format path name of web page         */
char *pagnam);                     /* page name (can start with "/" or "\")*/
                                   /* returns "" if combination too long   */
void
byeweb();                          /* say goodbye to a Web user w/msg      */
                                   /* same parameters as prfmsg()          */
                                   /* usrnum is an implicit input          */
                                   /* (do not use msgnum=NOMSG or PAMSG)   */
void
endwebd(                           /* terminate the WWW session            */
int complt);                       /* 1=finished, 0=aborted                */

void
datu2d(                            /* convert Unix time/date to DOS format */
unsigned long uxtime,              /*   Unix seconds since 1/1/70          */
unsigned short *dat,               /*   pointer to DOS packed date output  */
unsigned short *tim);              /*   pointer to DOS packed time output  */

void
datd2u(                            /* convert DOS time/date to Unix format */
unsigned short dat,                /*   DOS packed date                    */
unsigned short tim,                /*   DOS packed time                    */
unsigned long *uxtime);            /*   ptr, Unix secs since 1/1/70 output */

char *                             /*   returns pointer to formatted date  */
enchttpdate(                       /* Encode an HTTP date                  */
unsigned short dat,                /*   DOS packed date format             */
unsigned short tim);               /*   DOS packed time format             */

int                                /*   returns 1=ok, 0=bad format         */
dechttpdate(                       /* Decode an HTTP date                  */
char *dnt,                         /*   RFC 822, RFC 850, or ANSI C format */
unsigned short *dat,               /*   DOS packed date format             */
unsigned short *tim);              /*   DOS packed time format             */

